==> object-agent-0.6.0.tgz: package/src/traverse.js <==
import forOwn from './forOwn';
import isArray from './utility/isArray';
import isObject from './utility/isObject';

/**
 * Traverses a nested object.
 *
 * @example
 * ``` javascript
 * import { traverse } from 'object-agent';
 *
 * const thing = {
 *     a: [{
 *         b: 'c'
 *     }, {
 *         b: 'd'
 *     }],
 *     e: 'f
 * };
 *
 * traverse(thing, (path, value) => {
 *     console.log(path, value);
 * });
 * // => [], { a: [{ b: 'c' }, { b: 'd' }] }
 * // => ['a'], [{ b: 'c' }, { b: 'd' }]
 * // => ['a', 0], { b: 'c' }
 * // => ['a', 0, 'b'], 'c'
 * // => ['a', 1], { b: 'd' }
 * // => ['a', 1, 'b'], 'd'
 * // => ['e'], 'f'
 * ```
 *
 * @function traverse
 *
 * @arg {Object} object
 * @arg {Function} callback - Provides two args, path and value. If true is returned then stop traversing and return true.
 * @arg {Boolean} [isOptimistic=false] - If true then returning true in the callback will prevent going deeper down that branch, but will otherwise continue traversing.
 *
 * @returns {Boolean} true if the callback function returns a truthy value for any path; otherwise, false.
 */
export default (object, callback, isOptimistic) => {
	const processValue = (path, value) => {
		if (callback(path, value)) {
			return true;
		}
		if (isArray(value)) {
			return value.some((value, key) => processValue(path.concat(key), value));
		}
		if (isObject(value)) {
			return forOwn(value, (value, key) => processValue(path.concat(key), value));
		}
	};

	const processValueOptimistic = (path, value) => {
		let isCanceled = false;

		const loopCallback = (value, key) => {
			if (processValueOptimistic(path.concat(key), value)) {
				isCanceled = true;
			}
		};

		if (callback(path, value)) {
			isCanceled = true;
		}
		else if (isArray(value)) {
			value.some(loopCallback);
		}
		else if (isObject(value)) {
			forOwn(value, loopCallback);
		}

		return isCanceled;
	};

	return isOptimistic ? processValueOptimistic([], object) : processValue([], object);
};

==> object-agent-1.0.1.tgz: package/src/traverse.js <==
import forOwn from './forOwn';
import appendToPath from './utility/appendToPath';
import isArray from './utility/isArray';
import isObject from './utility/isObject';

/**
 * Traverses a nested object.
 *
 * @example
 * ``` javascript
 * import { traverse } from 'object-agent';
 *
 * const thing = {
 *     a: [{
 *         b: 'c'
 *     }, {
 *         b: 'd'
 *     }],
 *     e: 'f
 * };
 *
 * traverse(thing, (path, value) => {
 *     console.log(path, value);
 * });
 * // => [], { a: [{ b: 'c' }, { b: 'd' }] }
 * // => ['a'], [{ b: 'c' }, { b: 'd' }]
 * // => ['a', 0], { b: 'c' }
 * // => ['a', 0, 'b'], 'c'
 * // => ['a', 1], { b: 'd' }
 * // => ['a', 1, 'b'], 'd'
 * // => ['e'], 'f'
 * ```
 *
 * @function traverse
 *
 * @arg {Object} object
 * @arg {Function} callback - Provides two args, path and value. If true is returned then stop traversing and return true.
 * @arg {Boolean} [isOptimistic=false] - If true then returning true in the callback will prevent going deeper down that branch, but will otherwise continue traversing.
 *
 * @returns {Boolean} true if the callback function returns a truthy value for any path; otherwise, false.
 */
export default (object, callback, isOptimistic = false) => {
	const processValue = (path, value) => {
		let isCanceled = false;
		let result;

		const loopCallback = (value, key) => {
			if (processValue(appendToPath(path, key), value) === true) {
				if (!isOptimistic) {
					return true;
				}
				isCanceled = true;
			}
		};

		if (callback(path, value) === true) {
			return true;
		}

		if (isArray(value)) {
			result = value.some(loopCallback);
			if (!isOptimistic) {
				return result;
			}
		}
		else if (isObject(value)) {
			result = forOwn(value, loopCallback);
			if (!isOptimistic) {
				return result;
			}
		}

		return isCanceled;
	};

	return processValue('', object);
};
